(*
 * The MIT License (MIT)
 * 
 * Copyright (c) 2015 Cole Schlesinger
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *)


open Test_utils
open My_core

(* Run the tests. *)

let run (tests : (string * test_t) list) : unit = 
  let f ((name, test) : (string * test_t)) : unit = 
    begin try
      match test () with
      | Ok None       -> ()
      | Ok (Some msg) -> Printf.printf "%s failed: %s\n" name msg
      | Error e       -> Printf.printf "%s failed: %s\n" name e
    with
      e ->
        Printf.printf
          "%s failed with exception: \n%s\n"
          name
          (Printexc.to_string e);
          Printexc.print_backtrace stdout
    end in
  List.iter f tests


let () = run Test_syntax.tests
let () = run Test_semantics.tests
let () = run Test_compiler.tests
let () = run Test_equivalence.tests
