(*
 * The MIT License (MIT)
 * 
 * Copyright (c) 2015 Cole Schlesinger
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *)


open Equivalence
open Syntax
open My_core
open Test_utils

let mkTV (a : pred) : test_result =
  if pred_equiv a (normalize a) then
    Ok None
  else
    let msg = Printf.sprintf "TV failed: (%s) (%s)" (print_pred a) (print_nf (nf_of_pred a)) in
    Error msg

let mkTest (a : pred) (b : pred) (expected : bool) : (unit -> test_result) =
  fun () ->
  mkTV a >>= fun _ ->
  mkTV b >>= fun _ ->
  if (pred_equiv a b) = expected then
    Ok None
  else
    let msg =
      Printf.sprintf
        "expected: (%B),  got (%B)"
        expected
        (not expected) in
    Ok (Some msg)
   

let test_equivalence_1 =
  let a = And (Match ("SrcMAC", 1), Neg (Match ("DstMAC", 1))) in
  let b = And (Neg (Match ("DstMAC", 1)), Match ("SrcMAC", 1)) in
  let expected = true in
  mkTest a b expected

let test_equivalence_2 =
  let a = Or (Match ("SrcMAC", 1), Neg (Match ("DstMAC", 1))) in
  let b = And (Neg (Match ("DstMAC", 1)), Match ("SrcMAC", 1)) in
  let expected = false in
  mkTest a b expected

let test_equivalence_3 =
  let a = Or (Neg (Match ("SrcMAC", 1)), Neg (Match ("DstMAC", 1))) in
  let b = Neg (And (Match ("DstMAC", 1), (Match ("SrcMAC", 1)))) in
  let expected = true in
  mkTest a b expected

let test_equivalence_4 =
  let a = And (Neg (Match ("SrcMAC", 1)), Neg (Match ("DstMAC", 1))) in
  let b = Neg (Or (Match ("DstMAC", 1), (Match ("SrcMAC", 1)))) in
  let expected = true in
  mkTest a b expected


let tests =
  [ ("equivalence 1", test_equivalence_1)
  ; ("equivalence 2", test_equivalence_2)
  ; ("equivalence 3", test_equivalence_3)
  ; ("equivalence 4", test_equivalence_4)
  ]
