(*
 * The MIT License (MIT)
 * 
 * Copyright (c) 2015 Cole Schlesinger
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *)

open Syntax
open Semantics
open Compiler
open My_core
open Test_utils

(* Syntax tests. *)
let syntest_1 () : test_result =
  let p = Seq (Pred (Match ("InPort", 1)), (Modify ("OutPort", 2))) in
  infer_and_check p >>= fun _ -> Ok None

let syntest_2 () : test_result =
  let p = Seq (Pred (Match ("SrcMAC", 1)), (Modify ("SrcMAC", 2))) in
  infer_and_check p >>= fun _ -> Ok None

let nametest_1 () : test_result =
  let x1 = TableName.gen () in
  let x2 = TableName.gen () in
  if not (x1 = x2) then
    Ok None
  else
    let msg = "table gen gives same table name" in
    Ok (Some msg)

let extract_con_1 () : test_result =
  let p =
    (Modify ("OutPort", 1) <.> Modify ("DstMAC", 1))
    <.> Pred (Match ("SrcMAC", 2)) in
  let p_con = extract_concurrency p in
  let pks =
    mkPacketSet
      [ mkPacket [("SrcMAC", 1); ("DstMAC", 2); ("OutPort", 0)]
      ; mkPacket [("SrcMAC", 2); ("DstMAC", 1); ("OutPort", 0)]
      ] in
  setupTestEval pks FieldSet.empty p p_con

let tests : (string * test_t) list =
  [ ("syn test 1", syntest_1)
  ; ("syn test 2", syntest_2)
  ; ("name test 1", nametest_1)
  ; ("concurrency extraction 1", extract_con_1)
  ]
